@@ -60,7 +60,6 @@ def get_nparray_chunks(name, array, chunk_size=DEFAULT_FILE_CHUNK_SIZE):
         i += chunk_size
 
 
-
 class MapdlMath():
     """Abstract mapdl math class.  Created from a ``Mapdl`` instance.
 
@@ -103,7 +102,7 @@ class MapdlMath():
 
     def free(self):
         """Delete all vectors"""
-        self._mapdl.run("*FREE,ALL")
+        self._mapdl.run("*FREE,ALL", mute=True)
 
     def __repr__(self):
         return self._status
@@ -115,7 +114,7 @@ class MapdlMath():
     @property
     def _status(self):
         """Print out the status of all APDLMath Objects"""
-        return self._mapdl.run("*STATUS,MATH")
+        return self._mapdl.run("*STATUS,MATH", mute=False)
 
     def vec(self, size=0, dtype=np.double, init=None, name=None):
         """Create a vector
@@ -141,7 +140,7 @@ class MapdlMath():
 
         if not name:
             name = id_generator()
-            self._mapdl.run("*VEC," + name + "," + MYCTYPE[dtype] + ",ALLOC," + str(size))
+            self._mapdl.run(f"*VEC,{name},{MYCTYPE[dtype]},ALLOC,{size}", mute=True)
             return AnsVec(name, self._mapdl, dtype, init)
         else:
             return AnsVec(name, self._mapdl)
@@ -174,7 +173,8 @@ class MapdlMath():
 
         if not name:
             name = id_generator()
-            self._mapdl.run(f'*DMAT,{name},{MYCTYPE[dtype]},ALLOC,{nrow},{ncol}')
+            self._mapdl.run(f'*DMAT,{name},{MYCTYPE[dtype]},ALLOC,{nrow},{ncol}',
+                            mute=True)
             mat = AnsDenseMat(name, self._mapdl)
 
             if (init == "rand"):
@@ -353,9 +353,10 @@ class MapdlMath():
     def load_matrix_from_file(self, type=np.double, fname="file.full", matId="STIFF"):
         """Load a matrix from a file"""
         name = id_generator()
-        self._mapdl._log.info(f"Calling MAPDL to extract the %s matrix from %s",
+        self._mapdl._log.info("Calling MAPDL to extract the %s matrix from %s",
                               matId, fname)
-        self._mapdl.run(f"*SMAT,{name},{MYCTYPE[type]},IMPORT,FULL,{fname},{matId}")
+        self._mapdl.run(f"*SMAT,{name},{MYCTYPE[type]},IMPORT,FULL,{fname},{matId}",
+                        mute=True)
         return AnsSparseMat(name, self._mapdl)
 
     def stiff(self, dtype=np.double, fname="file.full"):
@@ -402,7 +403,8 @@ class MapdlMath():
         name = id_generator()
         self._mapdl._log.info("Call MAPDL to extract the %s vector from the file %s",
                               matId, fname)
-        self._mapdl.run(f"*VEC,{name},{MYCTYPE[dtype]},IMPORT,FULL,{fname},{matId}")
+        self._mapdl.run(f"*VEC,{name},{MYCTYPE[dtype]},IMPORT,FULL,{fname},{matId}",
+                        mute=True)
         return AnsVec(name, self._mapdl)
 
     def set_vec(self, vname, data):
@@ -483,14 +485,13 @@ class MapdlMath():
             cid = c.id
             algo = "DAMP"
 
-        self._mapdl.run("/SOLU")
-        self._mapdl.run("antype,modal")
-        self._mapdl.run("modopt," + algo + "," + str(nev) + "," + str(fmin) + "," + str(fmax))
+        self._mapdl.run("/SOLU", mute=True)
+        self._mapdl.run("antype,modal", mute=True)
+        self._mapdl.run(f"modopt,{algo},{nev},{fmin},{fmax}", mute=True)
         ev = self.vec()
 
         phistr = '' if not phi else phi.id
-        self._mapdl.run("*EIG," + k.id + "," + m.id + "," + cid + "," + ev.id + "," + phistr)
-
+        self._mapdl.run(f'*EIG,{k.id},{m.id},{cid},{ev.id},{phistr}', mute=True)
         return ev
 
     def dot(self, vec_a, vec_b):
@@ -714,7 +715,7 @@ class MapdlMath():
         if sym is True:
             flagsym = 'TRUE'
 
-        self._mapdl.run(f'*SMAT,{mname},D,ALLOC,CSR,{indptrname},{indxname},{dataname},{flagsym}')
+        self._mapdl.run(f'*SMAT,{mname},D,ALLOC,CSR,{indptrname},{indxname},{dataname},{flagsym}', mute=True)
 
 
 class ApdlMathObj:
@@ -731,7 +732,7 @@ class ApdlMathObj:
         return "APDLMath Object %s" % str(self.id)
 
     def __str__(self):
-        return self._mapdl.run("*PRINT," + self.id)
+        return self._mapdl.run("*PRINT," + self.id, mute=False)
 
     def copy(self):
         """Returns the name of the copy of this object"""
@@ -746,12 +747,11 @@ class ApdlMathObj:
             raise TypeError("Unknown obj type: operation aborted")
 
         # APDLMath cmd to COPY vin to vout
-        self._mapdl.run(acmd + "," + name + ",D,COPY," + self.id)
+        self._mapdl.run(acmd + "," + name + ",D,COPY," + self.id, mute=True)
         return name
 
     def _init(self, method):
-        cmd = "*INIT," + self.id + "," + method
-        self._mapdl.run(cmd)
+        self._mapdl.run(f"*INIT,{self.id},{method}", mute=True)
 
     def zeros(self):
         """Set all values of the vector to zero"""
@@ -767,7 +767,7 @@ class ApdlMathObj:
 
     def const(self, value):
         """Set all values of the vector to a constant"""
-        return self._init("CONST," + str(value))
+        return self._init(f"CONST,{value}")
 
     def norm(self, nrmtype="nrm2"):
         """Matrix or vector norm.
@@ -794,26 +794,26 @@ class ApdlMathObj:
         >>> nrm = mm.norm( m2)
         """
         val_name = 'py_val'
-        self._mapdl.run("*NRM," + self.id + "," + nrmtype + ",%s" % val_name)
+        self._mapdl.run(f"*NRM,{self.id},{nrmtype},{val_name}", mute=True)
         return self._mapdl.scalar_param(val_name)
 
     def axpy(self, op, val1, val2):
         """Perform the matrix operation: ``M2= v*M1 + w*M2``"""
         cmd = "*AXPY," + str(val1) + ",0," + op.id + "," + str(val2) + ",0," + self.id
-        self._mapdl._log.info(f">> Call Mapdl to perform AXPY operation")
-        self._mapdl.run(cmd)
+        self._mapdl._log.info(">> Call Mapdl to perform AXPY operation")
+        self._mapdl.run(cmd, mute=True)
         return self
 
     def __add__(self, op2):
         opout = self.copy()
         self._mapdl._log.info(">> Call Mapdl to perform AXPY operation")
-        self._mapdl.run(f"*AXPY,1,0,{op2.id},1,0,{opout.id}")
+        self._mapdl.run(f"*AXPY,1,0,{op2.id},1,0,{opout.id}", mute=True)
         return opout
 
     def __sub__(self, op2):
         opout = self.copy()
         self._mapdl._log.info("Call Mapdl to perform AXPY operation")
-        self._mapdl.run(f"*AXPY,-1,0,{op2.id},1,0,{opout.id}")
+        self._mapdl.run(f"*AXPY,-1,0,{op2.id},1,0,{opout.id}", mute=True)
         return opout
 
     def __iadd__(self, op):
@@ -824,14 +824,14 @@ class ApdlMathObj:
 
     def __imul__(self, val):
         self._mapdl._log.info("Call Mapdl to scale the object")
-        self._mapdl.run(f"*SCAL,{self.id},{val}")
+        self._mapdl.run(f"*SCAL,{self.id},{val}", mute=True)
         return self
 
     def __itruediv__(self, val):
         if val == 0:
             raise ZeroDivisionError('division by zero')
         self._mapdl._log.info("Call Mapdl to 1/scale the object")
-        self._mapdl.run(f"*SCAL,{self.id},{1/val}")
+        self._mapdl.run(f"*SCAL,{self.id},{1/val}", mute=True)
         return self
 
     @property
@@ -867,8 +867,7 @@ class AnsVec(ApdlMathObj):
     def __getitem__(self, num):
         if num < 0:
             raise ValueError('Negative indices not permitted')
-        cmd = "pyval=" + self.id + "(" + str(num+1) + ")"
-        self._mapdl.run(cmd)
+        self._mapdl.run(f"pyval={self.id}({num+1})", mute=True)
         return self._mapdl.scalar_param("pyval")
 
     def __mul__(self, vec2):
@@ -895,8 +894,7 @@ class AnsVec(ApdlMathObj):
         if not isinstance(vec, AnsVec):
             raise TypeError('Must be an Ansys vector object')
 
-        cmd = "*DOT," + self.id + "," + vec.id + ",py_val"
-        self._mapdl.run(cmd)
+        self._mapdl.run(f"*DOT,{self.id},{vec.id},py_val")
         return self._mapdl.scalar_param("py_val")
 
     def asarray(self):
@@ -960,7 +958,7 @@ class AnsMat(ApdlMathObj):
         return self._mapdl._mat_data(self.id)
 
     def __repr__(self):
-        return "APDLMath Matrix %d x %d" % (self.nrow, self.ncol)
+        return f"APDLMath Matrix {self.nrow} x {self.ncol}"
 
     def __mul__(self, vec):
         raise AttributeError('Array multiplication is not yet available.  '
@@ -993,21 +991,20 @@ class AnsMat(ApdlMathObj):
         info = self._mapdl._data_info(self.id)
         dtype = ANSYS_VALUE_TYPE[info.stype]
         if obj.type == ObjType.VEC:
-            self._mapdl.run(f"*VEC,{name},{MYCTYPE[dtype]},ALLOC,{info.size1}")
+            self._mapdl.run(f"*VEC,{name},{MYCTYPE[dtype]},ALLOC,{info.size1}",
+                            mute=True)
             objout = AnsVec(name, self._mapdl)
         else:
-            self._mapdl.run(f"*DMAT,{name},{MYCTYPE[dtype]},ALLOC,{info.size1},{info.size2}")
+            self._mapdl.run(f"*DMAT,{name},{MYCTYPE[dtype]},ALLOC,{info.size1},{info.size2}", mute=True)
             objout = AnsDenseMat(name, self._mapdl)
             
-        cmd = f"*MULT,{self.id},,{obj.id},,{name}"
         self._mapdl._log.info("Call Mapdl to perform MV Product")
-        self._mapdl.run(cmd)
+        self._mapdl.run(f"*MULT,{self.id},,{obj.id},,{name}", mute=True)
         return objout
 
     def __getitem__(self, num):
         name = id_generator()
-        cmd = "*VEC," + name + ",D,LINK," + self.id + "," + str(num+1)
-        self._mapdl.run(cmd)
+        self._mapdl.run(f"*VEC,{name},D,LINK,{self.id},{num+1}", mute=True)
         return AnsVec(name, self._mapdl)
 
     @property
@@ -1032,7 +1029,8 @@ class AnsMat(ApdlMathObj):
         dtype = ANSYS_VALUE_TYPE[info.stype]
         name = id_generator()
         self._mapdl._log.info("Call MAPDL to transpose")
-        self._mapdl.run(f"{objtype},{name},{MYCTYPE[dtype]},COPY,{self.id},TRANS")
+        self._mapdl.run(f"{objtype},{name},{MYCTYPE[dtype]},COPY,{self.id},TRANS",
+                        mute=True)
         if info.objtype == 2:
             mat = AnsDenseMat(name, self._mapdl)
         else:
@@ -1102,15 +1100,15 @@ class AnsSolver(ApdlMathObj):
                 Algo = "LAPACK"
             elif (M.type == ObjType.SMAT):
                 Algo = "DSP"
-        self._mapdl.run(f"*LSENGINE,{Algo},{self.id},{M.id}")
-        self._mapdl._log.info("Factorizing using the %s package", Algo)
-        self._mapdl.run("*LSFACTOR," + self.id)
+        self._mapdl.run(f"*LSENGINE,{Algo},{self.id},{M.id}", mute=True)
+        self._mapdl._log.info(f"Factorizing using the {Algo} package")
+        self._mapdl.run(f"*LSFACTOR,{self.id}", mute=True)
 
     def solve(self, B, X=None):
         if not X:
             X = B.copy()
         self._mapdl._log.info("Solving")
-        self._mapdl.run("*LSBAC," + self.id + "," + B.id + "," + X.id)
+        self._mapdl.run(f"*LSBAC,{self.id},{B.id},{X.id}", mute=True)
         return X
 
 
@@ -1122,7 +1120,7 @@ def rand(obj):
     obj : ansys.math.MapdlMath object
         MapdlMath object
     """
-    obj._mapdl.cmd("*INIT," + obj.id + ",RAND")
+    obj._mapdl.run(f"*INIT,{obj.id},RAND", mute=True)
 
 
 def solve(Mat, B, X=None, Algo=None):
@@ -1153,10 +1151,10 @@ def dot(vec1, vec2):
 
     """
     if (vec1.type != ObjType.VEC or vec2.type != ObjType.VEC):
-        raise Exception("Both objects must be ANSYS vectors")
+        raise TypeError("Both objects must be ANSYS vectors")
 
     mapdl = vec1._mapdl
-    mapdl.run("*DOT," + vec1.id + "," + vec2.id + ",py_val")
+    mapdl.run(f"*DOT,{vec1.id},{vec2.id},py_val", mute=True)
     return mapdl.scalar_param("py_val")
 
 
